/*
    This is 8051 Based IR Receiver for NEC protocol for PC Remote.
    Copyright (C) 2016  Rahul Kundu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
// Bootloader and Library Fnctions Starts Here //
#include <8051.h>
void setup();
void loop();
void main(){
	setup();
	while(1)loop();
}

// Bootloader and Library Fnctions Ends Here //
#define IR_PIN P3_7
#define LED P3_2
void serialInterrupt(void) __interrupt 4;
void repeatIR();
void sendIRData();
void sendSerialData(char sendData);
void processSerial(char receivedChar);
char hexToAscii(char hexVal);
void blinkLED();
unsigned int count = 0;
unsigned char index = 0;
unsigned int address = 0;
unsigned int command = 0;
__bit startBit = 0;

void setup(){
	PCON |=0x80;		//Achiving 19200 boud rate by doubling 9600
	TMOD |=0x20;		//Timer 1() 8-BIT auto-relod
	TH1=0xFD;		//9600 baud rate
	SCON=0x50;
	TR1=1;			//Start Timer1
	IE |=0x90;		//Enable Serial Interrupt,External Interrupt0
	blinkLED();
}
void loop(){
	count = 0;
	while(IR_PIN){
		count++;
	}; 
	if ((count > 630) && (count < 700)){ //Critical count 665
		startBit = 1; // set flag
		address = 0; // reser old address
		command = 0; // reset old command
	}
	else if ((count > 300) && (count < 360)){
		repeatIR();
	}
	else if(startBit){
		if(index < 16){ // reading address
			if((count >= 210) && (count <= 270)){ 
				if(index > 1){
					address = address << 1;
				}
				address |= 1;
			}else if ((count >= 40) && (count <= 100)){
				if (index > 1){
					address = address << 1;
				}
			}
		}
		else{ // reading command
			if((count >= 210) && (count <= 270)){ 
				if(index > 16){
					command = command << 1;
				}
				command |= 1;
			}else if ((count >= 40) && (count <= 100)){
				if (index > 16){
					command = command << 1;
				}
			}
		}
		index ++;
	}
	if(index >= 32){ // full message received
		sendIRData();
		index = 0;
		startBit = 0;
	}
	while (!IR_PIN); // wait until sensor flips
}
void serialInterrupt(void) __interrupt 4{		//Serial Interrupt
	char receivedData;
	if(RI==1){
		receivedData=SBUF;
		RI=0;
		//sendSerialData(receivedData);		//For echo
		processSerial(receivedData);
	}
}
void processSerial(char receivedChar){
	if(receivedChar=='N')
		LED=0;
	else if(receivedChar=='F')
		LED=1;
}
void sendSerialData(char sendData){
	SBUF=sendData;
	while(TI==0);	//wait for transmit
	TI=0;
}
void delay(unsigned int itime){
	unsigned int i,j;
	for(i=0;i<itime;i++)
		for(j=0;j<132;j++);
}
void repeatIR(){
	address = 0xFFFF;
	command = 0xFFFF;
	sendIRData();
}
void sendIRData(){
	sendSerialData('N');
	sendSerialData('E');
	sendSerialData('C');
	sendSerialData('-');
	
	if(((address >> 12) & 0x0F) != 0){
		sendSerialData(hexToAscii((address >> 12) & 0x0F));
	}
	if(((address >> 12) & 0x0F) != 0 || ((address >> 8) & 0x0F) != 0){
		sendSerialData(hexToAscii((address >> 8) & 0x0F));	
	}
	sendSerialData(hexToAscii((address >> 4) & 0x0F));
	sendSerialData(hexToAscii(address & 0x0F));
	sendSerialData(hexToAscii((command >> 12) & 0x0F));
	sendSerialData(hexToAscii((command >> 8) & 0x0F));
	sendSerialData(hexToAscii((command >> 4) & 0x0F));
	sendSerialData(hexToAscii(command & 0x0F));
	
	sendSerialData('\r');
	sendSerialData('\n');
}
char hexToAscii(char hexVal){
	char hexAsciiMap[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	hexVal = hexVal & 0x0F; //For Security of array out of bound
	return hexAsciiMap[hexVal];
}
void blinkLED(){
	LED = 0;
	delay(500);
	LED = 1;
	delay(500);
	LED = 0;
	delay(500);
	LED = 1;
}