/*
    This is 8051 Based IR Receiver for RC5 protocol for PC Remote.
    Copyright (C) 2016  Rahul Kundu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
// Bootloader and Library Fnctions Starts Here //
#include <8051.h>
void setup();
void loop();
void main(){
	setup();
	while(1)loop();
}

// Bootloader and Library Fnctions Ends Here //
#define IR_PIN P3_7
#define LED P3_2
void serialInterrupt(void) __interrupt 4;
void delayRC5(unsigned char p);
void sendSerialData(char sendData);
void sendIRData(char IRData);
void processSerial(char receivedChar);
char hexToAscii(char hexVal);
void blinkLED();
__bit flipBit = 0;

void setup(){
	PCON |=0x80;		//Achiving 19200 boud rate by doubling 9600
	TMOD |=0x20;		//Timer 1() 8-BIT auto-relod
	TH1=0xFD;		//9600 baud rate
	SCON=0x50;
	TR1=1;			//Start Timer1
	IE |=0x90;		//Enable Serial Interrupt,External Interrupt0
	blinkLED();
}
void loop(){
	unsigned char address = 0x00;
	unsigned char command = 0x00;
	unsigned char count=0x00;
	while(IR_PIN==1);	//Wait for first bit 
	delayRC5(7);		//Delay for 3.024ms
	flipBit=IR_PIN;		//Reading flip bit
	for(count=0;count<5;count++){
		delayRC5(4);	//Delay for 1.728ms
		address = address << 1;
		if(IR_PIN==1)
			address = address | 0x01;
	}
	for(count=0;count<6;count++){
		delayRC5(4);	//Delay for 1.728ms
		command = command << 1;
		if(IR_PIN==1)
			command = command | 0x01;
	}
	//P1 = command;
	if(address==0x00 || address==0x03){	//Cheking for valid address
		sendIRData(command);
	}
}
void serialInterrupt(void) __interrupt 4{		//Serial Interrupt
	char receivedData;
	if(RI==1){
		receivedData=SBUF;
		RI=0;
		//sendSerialData(receivedData);		//For echo
		processSerial(receivedData);
	}
}
void processSerial(char receivedChar){
	if(receivedChar=='N')
		LED=0;
	else if(receivedChar=='F')
		LED=1;
}
void sendSerialData(char sendData){
	SBUF=sendData;
	while(TI==0);	//wait for transmit
	TI=0;
}
void delayRC5(unsigned char r){ //delay for 432uS. For 3.024ms call 7 times and fro 1.728ms call 4 times 
	unsigned char p;
	unsigned int q;
	for(p=0;p<r;p++)
		for(q=0;q<53;q++);	
}
void delay(unsigned int itime){
	unsigned int i,j;
	for(i=0;i<itime;i++)
		for(j=0;j<132;j++);
}
void sendIRData(char IRData){
	char firstChar=hexToAscii((IRData >> 4) & 0x0F);
	char secondChar=hexToAscii(IRData & 0x0F);
	sendSerialData('R');
	sendSerialData('C');
	sendSerialData('5');
	sendSerialData('-');
	if(flipBit){
		sendSerialData('8');
		sendSerialData(firstChar);
		sendSerialData(secondChar);
	}
	else{
		if(firstChar!='0'){
			sendSerialData(firstChar);
			sendSerialData(secondChar);
		}
		else{
			sendSerialData(secondChar);
		}
	}
	sendSerialData('\r');
	sendSerialData('\n');
}
char hexToAscii(char hexVal){
	char asciiVal='i';
	switch(hexVal){
		case 0x01 : {asciiVal='1'; break;}
		case 0x02 : {asciiVal='2'; break;}
		case 0x03 : {asciiVal='3'; break;}
		case 0x04 : {asciiVal='4'; break;}
		case 0x05 : {asciiVal='5'; break;}
		case 0x06 : {asciiVal='6'; break;}
		case 0x07 : {asciiVal='7'; break;}
		case 0x08 : {asciiVal='8'; break;}
		case 0x09 : {asciiVal='9'; break;}
		case 0x0A : {asciiVal='A'; break;}
		case 0x0B : {asciiVal='B'; break;}
		case 0x0C : {asciiVal='C'; break;}
		case 0x0D : {asciiVal='D'; break;}
		case 0x0E : {asciiVal='E'; break;}
		case 0x0F : {asciiVal='F'; break;}
		case 0x00 : {asciiVal='0'; break;}
		default  :  asciiVal=0x00;
	}
	return asciiVal;
}
void blinkLED(){
	LED = 0;
	delay(500);
	LED = 1;
	delay(500);
	LED = 0;
	delay(500);
	LED = 1;
}