/*
    This is Desktop Application Program for PC Remote.
    Copyright (C) 2016  Rahul Kundu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import controlP5.*;
import processing.serial.*;
ControlP5 cp5;
RemoteRobot remoteRobotObj=new RemoteRobot();
ArrayList<String> comPorts = new ArrayList<String>();
static Serial remotePort;
ListBox portList;
static Textlabel textLabel;
static Textlabel textReceivedIR;
int commandBufferLen=0;
char[] commandBuffer = new char[15];
String connectedPort="";
int dropdownStartValue=10;
Boolean isSetUpReady=false;
static Boolean isLEDOn=false;
static boolean numMode=false;
static String[] shortcutArray = new String[10];
static XML[] keyConfig;
XML[] shortcutConfig;
long textLabelLastTime = 0;

void setup() {
  initConfig();
  initSerial();
  guiSetup();
  isSetUpReady=true;
  //Auto connect if connection port is configured in xml
  if(connectedPort != "" && comPorts.contains(connectedPort)){  
    connectSerial();
  }
}
void draw() {
  background(0,160,100);
  if ( millis() > textLabelLastTime + 3000 ) {
    textLabelLastTime = millis();
    textLabel.setText("SNR PC Remote");
  }
}

void initConfig(){
    XML config_xml = loadXML("configuration.xml"); 
    //Getting port name
    connectedPort = config_xml.getChildren("appConfig/serialPort")[0].getContent();
    // Getting all the element named by "remoteKey" as XML array object.
    keyConfig = config_xml.getChildren("keyMap/remoteKey");
    //Getting all shortcut
    shortcutConfig = config_xml.getChildren("shortcutMap/shortcut"); 	
    for(int k=0;k<shortcutArray.length;k++){
      for(int l=0;l<shortcutConfig.length;l++){
        if(k==shortcutConfig[l].getInt("id")){
          shortcutArray[k] = shortcutConfig[l].getContent();
        }
      }
    }
}
void initSerial(){
  String serialList[]=Serial.list();
  for(int port=0;port<serialList.length;port++){
    comPorts.add(serialList[port]);
  }
}
void controlEvent(ControlEvent theEvent) {
  textLabelLastTime = millis();
  if(isSetUpReady){
    int eventValue=(int)theEvent.value();
    if(eventValue==1){
      disConnectSerial();  
    }
    else if(eventValue==2){
      ledOnOff();
    }
    else if(eventValue==3){
      changeMode();
    }
    else if(eventValue==4){
      refreshList();
    }
    else if((eventValue-dropdownStartValue)>=0 && (eventValue-dropdownStartValue)<comPorts.size()){
      connectedPort=comPorts.get(eventValue-dropdownStartValue);
      connectSerial();
    }
  }
}
void serialEvent(Serial remotePort) {
  textLabelLastTime = millis();
  char data=(char)remotePort.read();
  if(commandBufferLen>=0 && commandBufferLen<commandBuffer.length){
    if ( data != '\r' && data != '\n' ) {
      commandBuffer[commandBufferLen]=data;
      commandBufferLen++;
    }
    else if(commandBufferLen > 0){
      StringBuffer result = new StringBuffer();
      for (int i = 0; i < commandBuffer.length; i++) {
         result.append( commandBuffer[i] );
      }
      String received=result.toString();
      remoteRobotObj.processKey(received.trim());
      commandBufferLen=0;
      for(int j=0;j<commandBuffer.length;j++){
         commandBuffer[j]=0;
      }
    }
   }
   else{
     //Preventing from array out of bound exception in case of gurbage value received
     commandBufferLen=0;  
     for(int j=0;j<commandBuffer.length;j++){
         commandBuffer[j]=0;
     }
   }
}
void connectSerial(){
  try{
    remotePort = new Serial(this, connectedPort, 19200);
    textLabel.setText("Remote is connected to "+connectedPort);
    cp5.controller("Light On/Off").show();
    cp5.controller("Change Mode").show();
    cp5.controller("Disconnect").show();
    cp5.controller("Received IR").show();
    cp5.controller("Refresh List").hide();
    portList.hide();
    ledOnOff();
    ledOnOff();
    ledOnOff();
    ledOnOff();
  }
  catch(Exception e){
    textLabel.setText("Error : Try Different Port");
  }
}
void disConnectSerial(){
  remotePort.stop();
  cp5.controller("Light On/Off").hide();
  cp5.controller("Change Mode").hide();
  cp5.controller("Disconnect").hide();
  cp5.controller("Received IR").hide();
  cp5.controller("Refresh List").show();
  portList.show();
}
void refreshList(){
      comPorts.clear();
      initSerial();
      portList.close();
      portList.clear();
      for(int i=0;i<comPorts.size();i++) {
        portList.addItem(comPorts.get(i),i+dropdownStartValue);
      }
}
public static void msDelay(int ms){
   try{    
    Thread.sleep(ms);
   }
   catch(Exception e){}
}
public static void ledOnOff(){
  if(isLEDOn==false){
    System.out.println("LED On");
    remotePort.write("LED-ON");
    remotePort.write('\r');
    isLEDOn=true;
    msDelay(200);
  }
  else{
    System.out.println("LED Off");
     remotePort.write("LED-OFF");
     remotePort.write('\r');
     isLEDOn=false;
     msDelay(200);
  }
}
public static void changeMode(){
    numMode=!numMode; 
    if(numMode){
      textLabel.setText("Remote is now Number Mode");
    }
    else {
      textLabel.setText("Remote is now Shortcut Mode");
    }
}
void guiSetup(){
  size(300,150);
  noStroke();
  cp5 = new ControlP5(this); 
  textLabel = cp5.addTextlabel("TextLabel").setText("Welcome to SNR Electronics").setPosition(15,5)
              .setColorValue(0xffffffff).setFont(createFont("Vardena",18));
  textReceivedIR = cp5.addTextlabel("Received IR").setText("").setPosition(15,130)
                   .setColorValue(0xff0000ff).setFont(createFont("Vardena",15));
  portList = cp5.addListBox("Port").setPosition(20, 60).setSize(50, 100);
  portList.actAsPulldownMenu(true);
  portList.setItemHeight(20);
  portList.setBarHeight(20);
  portList.captionLabel().style().marginTop = 5;
  
  for(int i=0;i<comPorts.size();i++) {
    portList.addItem(comPorts.get(i),i+dropdownStartValue);
  }
  cp5.addButton("Disconnect").setValue(1).setPosition(210,40).setSize(60,20);
  cp5.addButton("Light On/Off").setValue(2).setPosition(20,40).setSize(60,20);
  cp5.addButton("Change Mode").setValue(3).setPosition(120,40).setSize(60,20);
  cp5.addButton("Refresh List").setValue(4).setPosition(210,40).setSize(60,20);
  cp5.controller("Disconnect").hide();
  cp5.controller("Light On/Off").hide();
  cp5.controller("Change Mode").hide();
  cp5.controller("Received IR").hide();
}

