/*
    This is part of Desktop Application Program For PC Remote.
    Copyright (C) 2016  Rahul Kundu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.awt.AWTException;
import java.awt.Desktop;
import java.awt.Robot;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;

public class RemoteRobot{
    private static String prevAction="";
	private static String prevCommand="";
	private static long previousTimeStamp=0;
	private static int count_Alt_Tab=0;
	private static Robot robot;
        public RemoteRobot(){
            try {
                robot=new Robot();
            } catch (AWTException e) {
                e.printStackTrace();
            } 
        }
        public void processKey(String rawReceived){
			if(rawReceived.startsWith("NEC-") || rawReceived.startsWith("RC5-") || rawReceived.startsWith("RC6-") || rawReceived.startsWith("SON-") || rawReceived.startsWith("MAP-")){
				System.out.println(rawReceived);
				String receivedSplit[]=rawReceived.split("-");
				boolean repeat = false;
				String received=receivedSplit[1];

				if(receivedSplit[0].equalsIgnoreCase("NEC")){
					received=received.toString().trim();
					if(received.equalsIgnoreCase("FFFFFFFF")){
						repeat = true;
					}
					else{
						repeat = false;
						prevAction = getAction("NEC", receivedSplit[1]);
					}
				}
				else if(receivedSplit[0].equalsIgnoreCase("RC5")){
					  if(received.startsWith("8")){
						received=received.subSequence(1, 3).toString();
						if(received.startsWith("0")){
						  received=received.subSequence(1, 2).toString();
						}
					  }
					  if(prevCommand.equals(receivedSplit[1])){
						  repeat = true;
					  }
					  else{
						  repeat = false;
						  prevCommand = receivedSplit[1];
						  prevAction = getAction("RC5", received);
					  }
				}
				else if(receivedSplit[0].equalsIgnoreCase("RC6")){
					if(received.startsWith("104") || received.startsWith("4")){
					  if(received.startsWith("104")){
						received=received.subSequence(3, 5).toString();
					  }
					  else{
						received=received.subSequence(1, 3).toString();
					  }
					  if(prevCommand.equals(receivedSplit[1])){
						repeat = true;
					  }
					  else{
						repeat = false;
						prevCommand = receivedSplit[1];
						prevAction = getAction("RC6", received);
					  }
					}
				}
				else if(receivedSplit[0].equalsIgnoreCase("SON")){
					long currentTimeStamp=System.currentTimeMillis();
					long timeDiff=currentTimeStamp-previousTimeStamp;
					previousTimeStamp=currentTimeStamp;
					if(timeDiff<200){
						repeat = true;
					}
					else{
						repeat = false;
						prevAction = getAction("SON", receivedSplit[1]);
					}
				}
				else if(receivedSplit[0].equalsIgnoreCase("MAP")){
					//For Debug only
					prevAction = getAction("MAP", receivedSplit[1]);
				}
				keyPress(prevAction,repeat);
			}
        }
        private String getAction(String remoteType, String remoteCode){
			String actionString = "";
                        Remote.textReceivedIR.setText("Received IR : "+remoteType+"  --  " +remoteCode);
			for (int i = 0; i < Remote.keyConfig.length; i++) {
				if(Remote.keyConfig[i].getString("type").equalsIgnoreCase(remoteType)){
					if(Remote.keyConfig[i].getString("id").equalsIgnoreCase(remoteCode)){
						actionString = Remote.keyConfig[i].getContent();
						break;
					}
				} 
			}
                        if(actionString.equalsIgnoreCase("")){
                          actionString = "NOT_ASSIGNED"; 
                        }
			return actionString;
        }
	private void keyPress(String robotEvent,boolean isRepeat){
		if(robotEvent.equalsIgnoreCase("NUM0")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD0);
				robot.keyRelease(KeyEvent.VK_NUMPAD0);
				System.out.println("0");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[0]);
				System.out.println("Shortcut 0");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM1")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD1);
				robot.keyRelease(KeyEvent.VK_NUMPAD1);
				System.out.println("1");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[1]);
				System.out.println("Shortcut 1");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM2")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD2);
				robot.keyRelease(KeyEvent.VK_NUMPAD2);
				System.out.println("2");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[2]);
				System.out.println("Shortcut 2");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM3")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD3);
				robot.keyRelease(KeyEvent.VK_NUMPAD3);
				System.out.println("3");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[3]);
				System.out.println("Shortcut 3");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM4")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD4);
				robot.keyRelease(KeyEvent.VK_NUMPAD4);
				System.out.println("4");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[4]);
				System.out.println("Shortcut 4");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM5")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD5);
				robot.keyRelease(KeyEvent.VK_NUMPAD5);
				System.out.println("5");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[5]);
				System.out.println("Shortcut 5");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM6")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD6);
				robot.keyRelease(KeyEvent.VK_NUMPAD6);
				System.out.println("6");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[6]);
				System.out.println("Shortcut 6");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM7")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD7);
				robot.keyRelease(KeyEvent.VK_NUMPAD7);
				System.out.println("7");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[7]);
				System.out.println("Shortcut 7");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM8")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD8);
				robot.keyRelease(KeyEvent.VK_NUMPAD8);
				System.out.println("8");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[8]);
				System.out.println("Shortcut 8");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NUM9")){
			if(Remote.numMode){
				robot.keyPress(KeyEvent.VK_NUMPAD9);
				robot.keyRelease(KeyEvent.VK_NUMPAD9);
				System.out.println("9");
			}
			else if(!isRepeat){
				openLocation(Remote.shortcutArray[9]);
				System.out.println("Shortcut 9");
			}
		}
		else if(robotEvent.equalsIgnoreCase("ALT_TAB")){
			if(!isRepeat){
				robot.keyPress(KeyEvent.VK_ALT);
				for(int i=0;i<=count_Alt_Tab;i++){
					robot.keyPress(KeyEvent.VK_TAB);
					robot.keyRelease(KeyEvent.VK_TAB);
				}
				count_Alt_Tab++;
				if(count_Alt_Tab>20){
					count_Alt_Tab=0;
				}
				robot.keyRelease(KeyEvent.VK_ALT);
				System.out.println("ALT TAB");
			}
		}
		else if(robotEvent.equalsIgnoreCase("CTRL_RIGHT_ARROW")){
			robot.keyPress(KeyEvent.VK_CONTROL);
			robot.keyPress(KeyEvent.VK_RIGHT);
			robot.keyRelease(KeyEvent.VK_RIGHT);
			robot.keyRelease(KeyEvent.VK_CONTROL);
			System.out.println("CTRL RIGHT ARROW");
		}
		else if(robotEvent.equalsIgnoreCase("CTRL_LEFT_ARROW")){
			robot.keyPress(KeyEvent.VK_CONTROL);
			robot.keyPress(KeyEvent.VK_LEFT);
			robot.keyRelease(KeyEvent.VK_LEFT);
			robot.keyRelease(KeyEvent.VK_CONTROL);
			System.out.println("CTRL LEFT ARROW");
		}
		else if(robotEvent.equalsIgnoreCase("UP_ARROW")){
			robot.keyPress(KeyEvent.VK_UP);
			robot.keyRelease(KeyEvent.VK_UP);
			System.out.println("UP ARROW");
		}
		else if(robotEvent.equalsIgnoreCase("DOWN_ARROW")){
			robot.keyPress(KeyEvent.VK_DOWN);
			robot.keyRelease(KeyEvent.VK_DOWN);
			System.out.println("DOWN ARROW");
		}
		else if(robotEvent.equalsIgnoreCase("LEFT_ARROW")){
			robot.keyPress(KeyEvent.VK_LEFT);
			robot.keyRelease(KeyEvent.VK_LEFT);
			System.out.println("LEFT ARROW");
		}	
		else if(robotEvent.equalsIgnoreCase("RIGHT_ARROW")){
			robot.keyPress(KeyEvent.VK_RIGHT);
			robot.keyRelease(KeyEvent.VK_RIGHT);
			System.out.println("RIGHT ARROW");
		}
		else if(robotEvent.equalsIgnoreCase("ENTER_BTN")){
			if(!isRepeat){
				robot.keyPress(KeyEvent.VK_ENTER);
				robot.keyRelease(KeyEvent.VK_ENTER);
				System.out.println("ENTER");
			}
		}
		else if(robotEvent.equalsIgnoreCase("ALT_F4")){
			if(!isRepeat){
				robot.keyPress(KeyEvent.VK_ALT);
				robot.keyPress(KeyEvent.VK_F4);
				robot.keyRelease(KeyEvent.VK_F4);
				robot.keyRelease(KeyEvent.VK_ALT);
				System.out.println("ALT F4");
			}
		}
		else if(robotEvent.equalsIgnoreCase("BACK_SPACE")){
			if(!isRepeat){
				robot.keyPress(KeyEvent.VK_BACK_SPACE);
				robot.keyRelease(KeyEvent.VK_BACK_SPACE);
				System.out.println("BACK SPACE");
			}
		}
		else if(robotEvent.equalsIgnoreCase("CTRL_ALT_P")){
			if(!isRepeat){
				robot.keyPress(KeyEvent.VK_CONTROL);
				robot.keyPress(KeyEvent.VK_ALT);
				robot.keyPress(KeyEvent.VK_P);
				robot.keyRelease(KeyEvent.VK_P);
				robot.keyRelease(KeyEvent.VK_ALT);
				robot.keyRelease(KeyEvent.VK_CONTROL);
				System.out.println("CTRL ALT P");
			}
		}
		else if(robotEvent.equalsIgnoreCase("WINDOWS")){
			if(!isRepeat){
				robot.keyPress(524);
				robot.keyRelease(524);
				System.out.println("WINDOWS");
			}
		} 
		else if(robotEvent.equalsIgnoreCase("SPACE")){
			if(!isRepeat){
				robot.keyPress(KeyEvent.VK_SPACE);
				robot.keyRelease(KeyEvent.VK_SPACE);
				System.out.println("Space");
			}
		}
		else if(robotEvent.equalsIgnoreCase("MODE")){
			if(!isRepeat){
				Remote.changeMode();
				System.out.println("Mode Change");
			}
		}
		else if(robotEvent.equalsIgnoreCase("LED_ON_OFF")){
			if(!isRepeat){
				Remote.ledOnOff();
				System.out.println("LED On Off");
			}
		}
		else if(robotEvent.equalsIgnoreCase("NOT_ASSIGNED")){
			System.out.println("Not Assigned");
		}
		else {
			System.out.println("Key is not mapped in Config");
		}
  }
 
  private void openLocation(String location){
    try {
      Desktop.getDesktop().open(new File(location));
      Thread.sleep(500);
      robot.keyPress(KeyEvent.VK_RIGHT);
        robot.keyRelease(KeyEvent.VK_RIGHT);
      robot.keyPress(KeyEvent.VK_LEFT);
        robot.keyRelease(KeyEvent.VK_LEFT);
    } catch (Exception e) {
      System.out.println("Exception from RemoteRobot.java while opening location");
      System.out.println(e.getMessage());
    }
  }
  
}
