/*
    This is Remote Emulator for PC Remote.
    Copyright (C) 2016  Rahul Kundu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import controlP5.*;
import processing.serial.*;
ControlP5 cp5;
ArrayList<String> comPorts = new ArrayList<String>();
static Serial remotePort;
ListBox portList;
static Textlabel textLabel;
static Textlabel textReceivedIR;
int commandBufferLen=0;
char[] commandBuffer = new char[15];
String connectedPort="";
int dropdownStartValue=10;
Boolean isSetUpReady=false;

void setup() {
  initConfig();
  initSerial();
  guiSetup();
  isSetUpReady=true;
  // Auto connect code
  if(connectedPort!=""){            //If connection port is Fixed and Known
    connectSerial();
  }
  else if(comPorts.size()==1){    //Autometically connects if only one port is available
    connectedPort=comPorts.get(0);
    connectSerial();
  }
}
void draw() {
  background(200,50,10);
}

void initConfig(){
   
}
void controlEvent(ControlEvent theEvent) {
  if(isSetUpReady){
    int eventValue=(int)theEvent.value();
    if(eventValue==1){
      if(connectedPort.equalsIgnoreCase(""))
        textLabel.setText("Please select a port");
      else{
        connectSerial();
      }
    }
    else if(eventValue==2){
      remotePort.write("MAP-25");
      remotePort.write('\r');
    }
    else if(eventValue==3){
      remotePort.write("MAP-22");
      remotePort.write('\r');
    }
    else if((eventValue-dropdownStartValue)>=0 && (eventValue-dropdownStartValue)<comPorts.size()){
      connectedPort=comPorts.get(eventValue-dropdownStartValue);
      textLabel.setText(comPorts.get(eventValue-dropdownStartValue)+" Port is selected");
    }
  }
}
void serialEvent(Serial remotePort) {
  char data=(char)remotePort.read();
  if(commandBufferLen>=0 && commandBufferLen<commandBuffer.length){
    if ( data != '\r' && data != '\n' ) {
      commandBuffer[commandBufferLen]=data;
      commandBufferLen++;
    }
    else if(commandBufferLen > 0){
      StringBuffer result = new StringBuffer();
      for (int i = 0; i < commandBuffer.length; i++) {
         result.append( commandBuffer[i] );
      }
      String received=result.toString();
      textReceivedIR.setText(received.trim());
      commandBufferLen=0;
      for(int j=0;j<commandBuffer.length;j++){
         commandBuffer[j]=0;
      }
    }
   }
   else{
     commandBufferLen=0;  //Preventing from array out of bound exception in case of gurbage value received
     for(int j=0;j<commandBuffer.length;j++){
         commandBuffer[j]=0;
     }
   }
}
void connectSerial(){
  remotePort = new Serial(this, connectedPort, 19200);
  textLabel.setText("Remote is connected to "+connectedPort);
  cp5.controller("connect").hide();
  portList.hide();
  cp5.controller("Light On/Off").show();
  cp5.controller("Windows").show();
}
void initSerial(){
  String serialList[]=Serial.list();
  for(int port=0;port<serialList.length;port++){
    comPorts.add(serialList[port]);
  }
}
public static void msDelay(int ms){
   try{    
    Thread.sleep(ms);
  }
  catch(Exception e){}
}
void guiSetup(){
   size(300,150);
  noStroke();
  cp5 = new ControlP5(this); 
  textLabel = cp5.addTextlabel("TextLabel").setText("Rahul Remote Emulator").setPosition(15,5).setColorValue(0xffffffff).setFont(createFont("Vardena",18));
  textReceivedIR = cp5.addTextlabel("Received").setText("").setPosition(15,130).setColorValue(0xff0000ff).setFont(createFont("Vardena",15));
  portList = cp5.addListBox("Port").setPosition(20, 60).setSize(50, 100);
  portList.actAsPulldownMenu(true);
  portList.setItemHeight(20);
  portList.setBarHeight(20);
  portList.captionLabel().style().marginTop = 5;
  
  for(int i=0;i<comPorts.size();i++) {
    portList.addItem(comPorts.get(i),i+dropdownStartValue);
  }
  cp5.addButton("connect").setValue(1).setPosition(230,40).setSize(40,20);
  cp5.addButton("Light On/Off").setValue(2).setPosition(20,40).setSize(60,20);
  cp5.addButton("Windows").setValue(3).setPosition(230,40).setSize(45,20);
  cp5.controller("Light On/Off").hide();
  cp5.controller("Windows").hide();
}

